@@ -7,7 +7,8 @@ import onnx.helper
 import torch
 import torch.nn as nn
 import torch.onnx
-import torch.onnx.symbolic_registry as sym_reg
+import pytorch_pfn_extras
+import pytorch_pfn_extras.onnx.symbolic_registry as sym_reg
 
 
 class _AnnotationInit(object):
@@ -31,7 +32,8 @@ class _AnnotationInit(object):
     def setup(self, model: nn.Module, opset_ver: int) -> None:
         self._model: Optional[nn.Module] = model
         # dryrun to register every aten ops
-        sym_reg.register_version('', opset_ver)  # type: ignore[no-untyped-call]
+        if not pytorch_pfn_extras.requires("1.13.0"):
+            sym_reg.register_version('', opset_ver)  # type: ignore[no-untyped-call,attr-defined]
         self.opset_ver = opset_ver
 
     @property
